function [Simu] = MH_regression(y,X,nb_MCMC,graph)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Tutorial 2 : prior 4 (Beta prior for b1, NIP for the rest
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Inputs : 
%%%%% y contains the dependent variable (T * 1)
%%%%% X stands for the explanatory variables (T * k)
%%%%% nb_MCMC : number of MCMC iterations 
%%%%% graph : if =1, some convergence graphics are displayed at the end of
%%%%% the simulation (by default = 0)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Outputs :
%%%%% Simu is a structure containing :
%%%%% post_beta and post_sigma : MCMC Posterior draws of Beta and sigma^2
%%%%% beta_ols and sigma_ols are the OLS results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% By default, set the variable graph = 0.
if(nargin<4)
    graph=0;
end
%%%% Check if the variables y and X exhibit an appropriate size.
T = max(size(y));
dimension = min(size(X));
if(dimension~=size(X,2))
    X = X';
end
%%%% Compte the OLS estimator
beta_ols = (X'*X)\X'*y;
sigma_ols = (y-X*beta_ols)'*(y-X*beta_ols)/(T-dimension);

%%%%%%%%%%%
%%%% MCMC Initialisation
%%%%%%%%%%%
burn_in = ceil(0.3*nb_MCMC);
nb_post = nb_MCMC-burn_in;
beta_post = zeros(dimension,nb_post);
sigma_post = zeros(nb_post,1);

%%%%%%
%%% Computation of the variance of prior 3 in Tutorial 2
%%%%%
post_var_of_prior_3 = 0.1015^2;
%%%%%%
%%% Computation of the beta hyper-parameter 'a'
%%%%%
c = 0.3;
a = (c^2/post_var_of_prior_3-4)*(1/8);
%%%%%%
%%% Non-informative priors for the other beta's and for the variance
%%%%%
beta_0 = zeros(dimension,1);
M_0 = zeros(dimension);
M_0(2,2) = 1/post_var_of_prior_3;


disp('**********************************');
disp('***** BEGINNING OF THE MCMC ******');
disp('**********************************');
%%%%%%%%%%%%%%%
%%%% MCMC starting values
%%%%%%%%%%%%%%%
beta = beta_ols;
if(abs(beta_ols(2))>0.3)
    beta(2) = 0;
end
sigma = sigma_ols;
iter = 1;
accept = 0;
for i=1:nb_MCMC
    progressbar(i/nb_MCMC);
    if(i>burn_in)
        beta_post(:,iter) = beta;
        sigma_post(iter) = sigma;
        iter = iter +1;
    end
    %%%%%%%%%%%%%
    %%% Update of Sigma|beta
    %%%%%%%%%%%%%
    d = 0.5*((y-X*beta)'*(y-X*beta));
    c = 0.5*(T);
    inv_sigma = gamrnd(c,1/d);
    sigma = 1/inv_sigma;
    
    %%%%%%%%%%%%%
    %%% Update of beta|Sigma
    %%%%%%%%%%%%%
    Sigma_cond = M_0*sigma + X'*X;
    mu_cond = (X'*y + M_0*beta_0);
    Sigma_cond = inv(Sigma_cond);
    mu_cond = (sigma*M_0 + X'*X)\(mu_cond);
    Sigma_beta = Sigma_cond*sigma;
    beta_prop = mvnrnd(mu_cond,Sigma_beta)';
    
    if(abs(beta_prop(2))<0.3)
        log_like_stay = -0.5*T*log(2*pi*sigma)-0.5*(y-X*beta)'*(y-X*beta)/sigma;
        log_like_move = -0.5*T*log(2*pi*sigma)-0.5*(y-X*beta_prop)'*(y-X*beta_prop)/sigma;
        det_Sig_cond = det(Sigma_beta);
        inv_Sigma_cond = inv(Sigma_beta);
        log_prop_stay = -0.5*dimension*log(2*pi)-0.5*log(det_Sig_cond)-0.5*(beta-mu_cond)'*inv_Sigma_cond*(beta-mu_cond);
        log_prop_move = -0.5*dimension*log(2*pi)-0.5*log(det_Sig_cond)-0.5*(beta_prop-mu_cond)'*inv_Sigma_cond*(beta_prop-mu_cond);
        prior_stay = gammaln(2*a)-2*gammaln(a)-(2*a-1)*log(2*c)+(a-1)*log((beta(2)+c)*(c-beta(2)));
        prior_move = gammaln(2*a)-2*gammaln(a)-(2*a-1)*log(2*c)+(a-1)*log((beta_prop(2)+c)*(c-beta_prop(2)));
        if(exp(log_like_move+prior_move-log_prop_move-log_like_stay-prior_stay+log_prop_stay)>rand())
            beta = beta_prop;
            accept = accept+1;
        end
    end
    
end
%%%%%%%%%%%%%
%%% Gathering all the statistics into the structure Simu
%%%%%%%%%%%%%
Simu.post_beta = beta_post;
Simu.post_sigma = sigma_post';

Simu.post_mean_beta = mean(beta_post,2);
Simu.post_std_beta = std(beta_post,0,2);
Simu.post_mean_sigma2 = mean(sigma_post,1);

Simu.beta_ols = beta_ols;
Simu.sigma_ols = sigma_ols;
Simu.accept = accept/nb_MCMC;
Simu.prior_a_c = [a c];
Simu.test_Geweke = Geweke_test(beta_post);

%%%%%%%%%%%%%
%%% Testing for the convergence by cusum plot and ACF
%%%%%%%%%%%%%
[Simu.cusum] = cusum_plot(beta_post,graph);
if(graph==1)
    figure;
    for z=1:dimension
        subplot(ceil(0.5*dimension),2,z),autocorr(beta_post(z,:),200),title(['Sample ACF of \beta : ' num2str(z)]);
    end
    figure;
    autocorr(sigma_post,200),title('Sample ACF of \sigma^2');
end
    